import json
import tensorflow as tf
import io
import zipfile
import pandas as pd
import requests
import matplotlib.pyplot as plt
import numpy as np


def transpose_dataframe(data):
    df = pd.DataFrame.from_dict(data)
    prices = df.values
    new = np.zeros(prices.T.shape)
    for i in range(prices.shape[0]):
        row = prices[i, :]
        for j in range(prices.shape[1]):
            value = float(row[-j-1])
            new[j, i] = value
    return new


# YOUR FUNCTION DOES NOT HANDLE ALL THE REQUIRED PARAMETERS NEED TO CHANGE!!!!!
def get_time_series_data(function, symbol, interval, api_key):
    if function == 'TIME_SERIES_INTRADAY':
        base_url = 'https://www.alphavantage.co/query?'
        output_size = 'full'

        response = requests.get(
            f'{base_url}function={function}&symbol={symbol}&interval={interval}&outputsize={output_size}&apikey={api_key}')
        response = response.json()
        data = response[f'Time Series ({interval})']
        df = pd.DataFrame.from_dict(data)
        prices = df.iloc[0, :].values
        prices = [float(prices[-i-1]) for i in range(1, len(prices))]
        prices = pd.DataFrame(prices, columns=[symbol])

    elif function == 'TIME_SERIES_DAILY':
        base_url = 'https://www.alphavantage.co/query?'
        output_size = 'full'

        response = requests.get(
            f'{base_url}function={function}&symbol={symbol}&outputsize={output_size}&apikey={api_key}')
        response = response.json()
        data = response['Time Series (Daily)']
        new = transpose_dataframe(data)

        prices = pd.DataFrame(new, columns=(
            ['daily open', 'daily high', 'daily low', 'daily close', 'daily volume']))

    elif function == 'TIME_SERIES_DAILY_ADJUSTED':
        base_url = 'https://www.alphavantage.co/query?'
        output_size = 'full'

        response = requests.get(
            f'{base_url}function={function}&symbol={symbol}&outputsize={output_size}&apikey={api_key}')
        response = response.json()
        data = response['Time Series (Daily)']
        new = transpose_dataframe(data)

        prices = pd.DataFrame(new, columns=(['daily open', 'daily high', 'daily low', 'daily close',
                                             'daily adjusted close', 'daily volume', 'divident amount', 'split coefficient']))

    elif function == 'TIME_SERIES_WEEKLY':
        base_url = 'https://www.alphavantage.co/query?'
        output_size = 'full'

        response = requests.get(
            f'{base_url}function={function}&symbol={symbol}&apikey={api_key}')
        response = response.json()
        data = response['Weekly Time Series']
        new = transpose_dataframe(data)

        prices = pd.DataFrame(new, columns=(
            ['weekly open', 'weekly high', 'weekly low', 'weekly close', 'weekly volume']))

    elif function == 'TIME_SERIES_WEEKLY_ADJUSTED':
        base_url = 'https://www.alphavantage.co/query?'
        output_size = 'full'

        response = requests.get(
            f'{base_url}function={function}&symbol={symbol}&apikey={api_key}')
        response = response.json()
        data = response['Weekly Adjusted Time Series']
        new = transpose_dataframe(data)

        prices = pd.DataFrame(new, columns=(['weekly open', 'weekly high', 'weekly low',
                                             'weekly close', 'weekly adjusted close', 'weekly volume', 'dividend amount']))

    elif function == 'TIME_SERIES_MONTHLY':
        base_url = 'https://www.alphavantage.co/query?'
        output_size = 'full'

        response = requests.get(
            f'{base_url}function={function}&symbol={symbol}&apikey={api_key}')
        response = response.json()
        data = response['Monthly Time Series']
        new = transpose_dataframe(data)

        prices = pd.DataFrame(new, columns=(
            ['monthly open', 'monthly high', 'monthly low', 'monthly close', 'monthly volume']))

    elif function == 'TIME_SERIES_MONTHLY_ADJUSTED':
        base_url = 'https://www.alphavantage.co/query?'
        output_size = 'full'

        response = requests.get(
            f'{base_url}function={function}&symbol={symbol}&apikey={api_key}')
        response = response.json()
        data = response['Monthly Adjusted Time Series']
        new = transpose_dataframe(data)

        prices = pd.DataFrame(new, columns=(['monthly open', 'monthly high', 'monthly low',
                                             'monthly close', 'monthly adjusted close', "monthly volume", "dividend amount"]))
    return prices



import numpy as np
import matplotlib.pyplot as plt
import requests
import pandas as pd
import zipfile
import json
import tensorflow as tf
from time_series import transpose_dataframe


def get_response_as_df(response, indicator):
    response = response.json()
    response = response[f'Technical Analysis: {indicator}']
    df = pd.DataFrame.from_dict(response)
    indicator_df = transpose_dataframe(df)
    indicator_df = pd.DataFrame(indicator_df, columns=([list(df.index)]))
    # other_columns = ['Value' for i in range(indicator_df.shape[1]-1)]
    # indicator_df = pd.DataFrame(indicator_df, columns = ([f'{indicator}'] + other_columns))
    return indicator_df


def get_technical_indicator(ticker, api_key, time_interval='daily', window_size=15, series_type='close', indicator='EMA'):
    indicator_df = 0
    base_url = 'https://www.alphavantage.co/query?'
    if indicator == 'SMA':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'RSI':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'EMA':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'WMA':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'DEMA':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'TEMA':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'TRIMA':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'KAMA':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'VWAP':
        default_time_interval = '15min'
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={default_time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'MAMA':
        fast_limit = 0.01
        slow_limit = 0.01
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&fast_limit={fast_limit}&slow_limit{slow_limit}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'T3':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'MACD':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'MACDEXT':
        fast_limit = 0.01
        slow_limit = 0.01
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&fast_limit={fast_limit}&slow_limit{slow_limit}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'STOCH':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'STOCHF':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'STOCHRSI':
        # function=STOCHRSI&symbol=IBM&interval=daily&time_period=10&series_type=close&fastkperiod=6&fastdmatype=1&apikey=demo
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&fastkperiod=6&fastdmatype=1&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'WILLR':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'ADX':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'ADXR':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'APO':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&series_type={series_type}&fast_period=10&matype=1&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'PPO':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&series_type={series_type}&fast_period=10&matype=1&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'MOM':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'BOP':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'CCI':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'CMO':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'ROC':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'ROCR':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'AROON':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'AROONOSC':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'MFI':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'TRIX':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'ULTOSC':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&timeperiod1=8&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'DX':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'MINUS_DI':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'PLUS_DI':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'MINUS_DM':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'PLUS_DM':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'BBANDS':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&nbdevup=3&nbdevdn=3&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'MIDPOINT':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'MIDPRICE':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'SAR':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&acceleration=0.05&maximum=0.25&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'TRANGE':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'ATR':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'NATR':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&time_period={window_size}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'AD':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&apikey={api_key}')
        response = response.json()
        response = response[f'Technical Analysis: Chaikin A/D']
        df = pd.DataFrame.from_dict(response)
        indicator_df = transpose_dataframe(df)
        indicator_df = pd.DataFrame(indicator_df, columns=([list(df.index)]))
    elif indicator == 'ADOSC':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&fastperiod=5&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'OBV':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'HT_TRENDLINE':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'HT_SINE':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'HT_TRENDMODE':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'HT_DCPERIOD':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'HT_DCPHASE':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    elif indicator == 'HT_PHASOR':
        response = requests.get(
            f'{base_url}function={indicator}&symbol={ticker}&interval={time_interval}&series_type={series_type}&apikey={api_key}')
        indicator_df = get_response_as_df(response, indicator)
    else:
        print(f'No data was returned for {indicator}')

    test = np.sum(np.array(indicator_df))
    if test > 0:
        return indicator_df
    else:
        pass
